@@ -1,4 +1,4 @@
-# Copyright 2018-2020 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,12 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import sys
-from streamlit.cli import main
+from streamlit.web.cli import main
 
 if __name__ == "__main__":
-    # So that the Streamlit server sees the same command line string
-    # whether streamlit is called directly or via `python -m streamlit`.
-    sys.argv[0] = "streamlit"
-
-    main()
+    # Set prog_name so that the Streamlit server sees the same command line
+    # string whether streamlit is called directly or via `python -m streamlit`.
+    main(prog_name="streamlit")
