@@ -263,6 +263,7 @@ def _global_development_mode() -> bool:
         not env_util.is_pex()
         and "site-packages" not in __file__
         and "dist-packages" not in __file__
+        and "__pypackages__" not in __file__
     )
 
 
@@ -355,7 +356,7 @@ def _logger_message_format() -> str:
     [Python's documentation](https://docs.python.org/2.6/library/logging.html#formatter-objects)
     for available attributes.
 
-    Default: None
+    Default: "%(asctime)s %(message)s"
     """
     if get_option("global.developmentMode"):
         from streamlit.logger import DEFAULT_LOG_MESSAGE
@@ -365,6 +366,21 @@ def _logger_message_format() -> str:
         return "%(asctime)s %(message)s"
 
 
+_create_option(
+    "logger.enableRich",
+    description="""
+        Controls whether uncaught app exceptions are logged via the rich library.
+
+        If True and if rich is installed, exception tracebacks will be logged with syntax highlighting and formatting.
+        Rich tracebacks are easier to read and show more code than standard Python tracebacks.
+
+        If set to False, the default Python traceback formatting will be used.""",
+    default_val=False,
+    visibility="hidden",
+    type_=bool,
+    scriptable=True,
+)
+
 # Config Section: Client #
 
 _create_section("client", "Settings for scripts that use Streamlit.")
@@ -448,6 +464,19 @@ _create_option(
     type_=bool,
 )
 
+_create_option(
+    "runner.fastReruns",
+    description="""
+        Handle script rerun requests immediately, rather than waiting for
+        script execution to reach a yield point. Enabling this will
+        make Streamlit much more responsive to user interaction, but it can
+        lead to race conditions in apps that mutate session_state data outside
+        of explicit session_state assignment statements.
+    """,
+    default_val=False,
+    type_=bool,
+)
+
 # Config Section: Server #
 
 _create_section("server", "Settings for the Streamlit server")
@@ -625,9 +654,9 @@ def _server_max_message_size() -> int:
 def _server_enable_websocket_compression() -> bool:
     """Enables support for websocket compression.
 
-    Default: true
+    Default: false
     """
-    return True
+    return False
 
 
 # Config Section: Browser #
@@ -676,6 +705,10 @@ def _browser_server_port() -> int:
 
 # Config Section: UI #
 
+# NOTE: We currently hide the ui config section in the `streamlit config show`
+# output as all of its options are hidden. If a non-hidden option is eventually
+# added, the section should be unhidden by removing it from the `SKIP_SECTIONS`
+# set in config_util.show_config.
 _create_section("ui", "Configuration of UI elements displayed in the browser.")
 
 _create_option(
@@ -687,6 +720,15 @@ _create_option(
     """,
     default_val=False,
     type_=bool,
+    visibility="hidden",
+)
+
+_create_option(
+    "ui.hideSidebarNav",
+    description="Flag to hide the sidebar page navigation component.",
+    default_val=False,
+    type_=bool,
+    visibility="hidden",
 )
 
 
@@ -869,7 +911,7 @@ def _set_option(key: str, value: Any, where_defined: str) -> None:
         LOGGER = get_logger(__name__)
 
         LOGGER.warning(
-            f' "{key}" is not a valid config option. If you previously had this config option set, it may have been removed.'
+            f'"{key}" is not a valid config option. If you previously had this config option set, it may have been removed.'
         )
 
     else:
