@@ -13,15 +13,32 @@
 # limitations under the License.
 
 """Allows us to create and absorb changes (aka Deltas) to elements."""
-from typing import Optional, Iterable
+from typing import (
+    Any,
+    Callable,
+    cast,
+    Dict,
+    Hashable,
+    Optional,
+    overload,
+    Iterable,
+    NoReturn,
+    Tuple,
+    Type,
+    TypeVar,
+    TYPE_CHECKING,
+    Union,
+)
+
+from typing_extensions import Final, Literal
 
 import streamlit as st
-from streamlit import cursor, caching
-from streamlit import legacy_caching
+from streamlit import cursor
+from streamlit.runtime import caching, legacy_caching
 from streamlit import type_util
 from streamlit import util
 from streamlit.cursor import Cursor
-from streamlit.script_run_context import get_script_run_ctx
+from streamlit.runtime.scriptrunner import get_script_run_ctx
 from streamlit.errors import StreamlitAPIException
 from streamlit.errors import NoSessionContext
 from streamlit.proto import Block_pb2
@@ -32,6 +49,7 @@ from streamlit.logger import get_logger
 from streamlit.elements.balloons import BalloonsMixin
 from streamlit.elements.button import ButtonMixin
 from streamlit.elements.markdown import MarkdownMixin
+from streamlit.elements.heading import HeadingMixin
 from streamlit.elements.text import TextMixin
 from streamlit.elements.alert import AlertMixin
 from streamlit.elements.json import JsonMixin
@@ -59,12 +77,13 @@ from streamlit.elements.color_picker import ColorPickerMixin
 from streamlit.elements.file_uploader import FileUploaderMixin
 from streamlit.elements.select_slider import SelectSliderMixin
 from streamlit.elements.slider import SliderMixin
+from streamlit.elements.snow import SnowMixin
 from streamlit.elements.image import ImageMixin
 from streamlit.elements.pyplot import PyplotMixin
 from streamlit.elements.write import WriteMixin
 from streamlit.elements.layouts import LayoutsMixin
 from streamlit.elements.form import FormMixin, FormData, current_form_id
-from streamlit.state.widgets import NoValue
+from streamlit.runtime.state import NoValue
 
 # DataFrame elements come in two flavors: "Legacy" and "Arrow".
 # We select between them with the DataFrameElementSelectorMixin.
@@ -76,22 +95,32 @@ from streamlit.elements.legacy_altair import LegacyAltairMixin
 from streamlit.elements.legacy_vega_lite import LegacyVegaLiteMixin
 from streamlit.elements.dataframe_selector import DataFrameSelectorMixin
 
-LOGGER = get_logger(__name__)
+if TYPE_CHECKING:
+    from numpy import typing as npt
+    from pandas import DataFrame, Series
+    from google.protobuf.message import Message
+    from streamlit.elements.arrow import Data
+
+
+LOGGER: Final = get_logger(__name__)
 
 # Save the type built-in for when we override the name "type".
 _type = type
 
-MAX_DELTA_BYTES = 14 * 1024 * 1024  # 14MB
+MAX_DELTA_BYTES: Final[int] = 14 * 1024 * 1024  # 14MB
 
 # List of Streamlit commands that perform a Pandas "melt" operation on
 # input dataframes.
-DELTA_TYPES_THAT_MELT_DATAFRAMES = ("line_chart", "area_chart", "bar_chart")
-ARROW_DELTA_TYPES_THAT_MELT_DATAFRAMES = (
+DELTA_TYPES_THAT_MELT_DATAFRAMES: Final = ("line_chart", "area_chart", "bar_chart")
+ARROW_DELTA_TYPES_THAT_MELT_DATAFRAMES: Final = (
     "arrow_line_chart",
     "arrow_area_chart",
     "arrow_bar_chart",
 )
 
+Value = TypeVar("Value")
+DG = TypeVar("DG", bound="DeltaGenerator")
+
 
 class DeltaGenerator(
     AlertMixin,
@@ -106,6 +135,7 @@ class DeltaGenerator(
     FileUploaderMixin,
     FormMixin,
     GraphvizMixin,
+    HeadingMixin,
     HelpMixin,
     IframeMixin,
     ImageMixin,
@@ -124,6 +154,7 @@ class DeltaGenerator(
     SelectboxMixin,
     SelectSliderMixin,
     SliderMixin,
+    SnowMixin,
     JsonMixin,
     TextMixin,
     TextWidgetsMixin,
@@ -174,7 +205,7 @@ class DeltaGenerator(
         cursor: Optional[Cursor] = None,
         parent: Optional["DeltaGenerator"] = None,
         block_type: Optional[str] = None,
-    ):
+    ) -> None:
         """Inserts or updates elements in Streamlit apps.
 
         As a user, you should never initialize this object by hand. Instead,
@@ -224,13 +255,18 @@ class DeltaGenerator(
     def __repr__(self) -> str:
         return util.repr_(self)
 
-    def __enter__(self):
+    def __enter__(self) -> None:
         # with block started
         ctx = get_script_run_ctx()
         if ctx:
             ctx.dg_stack.append(self)
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(
+        self,
+        type: Any,
+        value: Any,
+        traceback: Any,
+    ) -> Literal[False]:
         # with block ended
         ctx = get_script_run_ctx()
         if ctx is not None:
@@ -265,14 +301,14 @@ class DeltaGenerator(
         """
         return self._parent._main_dg if self._parent else self
 
-    def __getattr__(self, name):
+    def __getattr__(self, name: str) -> Callable[..., NoReturn]:
         import streamlit as st
 
         streamlit_methods = [
             method_name for method_name in dir(st) if callable(getattr(st, method_name))
         ]
 
-        def wrapper(*args, **kwargs):
+        def wrapper(*args: Any, **kwargs: Any) -> NoReturn:
             if name in streamlit_methods:
                 if self._root_container == RootContainer.SIDEBAR:
                     message = (
@@ -320,6 +356,10 @@ class DeltaGenerator(
     def _is_top_level(self) -> bool:
         return self._provided_cursor is None
 
+    @property
+    def id(self) -> str:
+        return str(id(self))
+
     def _get_delta_path_str(self) -> str:
         """Returns the element's delta path as a string like "[0, 2, 3, 1]".
 
@@ -334,15 +374,75 @@ class DeltaGenerator(
         dg = self._active_dg
         return str(dg._cursor.delta_path) if dg._cursor is not None else "[]"
 
+    @overload
+    def _enqueue(  # type: ignore[misc]
+        self,
+        delta_type: str,
+        element_proto: "Message",
+        return_value: None,
+        last_index: Optional[Hashable] = None,
+        element_width: Optional[int] = None,
+        element_height: Optional[int] = None,
+    ) -> "DeltaGenerator":
+        ...
+
+    @overload
+    def _enqueue(  # type: ignore[misc]
+        self,
+        delta_type: str,
+        element_proto: "Message",
+        return_value: Type[NoValue],
+        last_index: Optional[Hashable] = None,
+        element_width: Optional[int] = None,
+        element_height: Optional[int] = None,
+    ) -> None:
+        ...
+
+    @overload
+    def _enqueue(  # type: ignore[misc]
+        self,
+        delta_type: str,
+        element_proto: "Message",
+        return_value: Value,
+        last_index: Optional[Hashable] = None,
+        element_width: Optional[int] = None,
+        element_height: Optional[int] = None,
+    ) -> Value:
+        ...
+
+    @overload
     def _enqueue(
         self,
-        delta_type,
-        element_proto,
-        return_value=None,
-        last_index=None,
-        element_width=None,
-        element_height=None,
-    ):
+        delta_type: str,
+        element_proto: "Message",
+        return_value: None = None,
+        last_index: Optional[Hashable] = None,
+        element_width: Optional[int] = None,
+        element_height: Optional[int] = None,
+    ) -> "DeltaGenerator":
+        ...
+
+    @overload
+    def _enqueue(
+        self,
+        delta_type: str,
+        element_proto: "Message",
+        return_value: Union[None, Type[NoValue], Value] = None,
+        last_index: Optional[Hashable] = None,
+        element_width: Optional[int] = None,
+        element_height: Optional[int] = None,
+    ) -> Union["DeltaGenerator", None, Value]:
+        ...
+
+    def _enqueue(
+        self,
+        delta_type: str,
+        element_proto: "Message",
+        return_value: Union[None, Type[NoValue], Value] = None,
+        last_index: Optional[Hashable] = None,
+        element_width: Optional[int] = None,
+        element_height: Optional[int] = None,
+    ) -> Union["DeltaGenerator", None, Value]:
         """Create NewElement delta, fill it, and enqueue it.
 
         Parameters
@@ -428,9 +528,21 @@ class DeltaGenerator(
             # no-op from the point of view of the app.
             output_dg = dg
 
+        # Save message for replay if we're called from within @st.memo or @st.singleton
+        caching.save_element_message(
+            delta_type,
+            element_proto,
+            invoked_dg_id=self.id,
+            used_dg_id=dg.id,
+            returned_dg_id=output_dg.id,
+        )
+
         return _value_or_dg(return_value, output_dg)
 
-    def _block(self, block_proto=Block_pb2.Block()) -> "DeltaGenerator":
+    def _block(
+        self,
+        block_proto: Block_pb2.Block = Block_pb2.Block(),
+    ) -> "DeltaGenerator":
         # Operate on the active DeltaGenerator, in case we're in a `with` block.
         dg = self._active_dg
 
@@ -475,9 +587,22 @@ class DeltaGenerator(
         dg._cursor.get_locked_cursor(last_index=None)
         _enqueue_message(msg)
 
+        caching.save_block_message(
+            block_proto,
+            invoked_dg_id=self.id,
+            used_dg_id=dg.id,
+            returned_dg_id=block_dg.id,
+        )
+
         return block_dg
 
-    def _legacy_add_rows(self, data=None, **kwargs):
+    def _legacy_add_rows(
+        self: DG,
+        data: "Data" = None,
+        **kwargs: Union[
+            "DataFrame", "npt.NDArray[Any]", Iterable[Any], Dict[Hashable, Any], None
+        ],
+    ) -> Optional[DG]:
         """Concatenate a dataframe to the bottom of the current one.
 
         Parameters
@@ -549,8 +674,8 @@ class DeltaGenerator(
             )
 
         # When doing _legacy_add_rows on an element that does not already have data
-        # (for example, st._legacy_ine_chart() without any args), call the original
-        # st._legacy_foo() element with new data instead of doing an _legacy_add_rows().
+        # (for example, st._legacy_line_chart() without any args), call the original
+        # st._legacy_foo() element with new data instead of doing a _legacy_add_rows().
         if (
             self._cursor.props["delta_type"] in DELTA_TYPES_THAT_MELT_DATAFRAMES
             and self._cursor.props["last_index"] is None
@@ -561,7 +686,7 @@ class DeltaGenerator(
             st_method_name = "_legacy_" + self._cursor.props["delta_type"]
             st_method = getattr(self, st_method_name)
             st_method(data, **kwargs)
-            return
+            return None
 
         data, self._cursor.props["last_index"] = _maybe_melt_data_for_add_rows(
             data, self._cursor.props["delta_type"], self._cursor.props["last_index"]
@@ -582,7 +707,13 @@ class DeltaGenerator(
 
         return self
 
-    def _arrow_add_rows(self, data=None, **kwargs):
+    def _arrow_add_rows(
+        self: DG,
+        data: "Data" = None,
+        **kwargs: Union[
+            "DataFrame", "npt.NDArray[Any]", Iterable[Any], Dict[Hashable, Any], None
+        ],
+    ) -> Optional[DG]:
         """Concatenate a dataframe to the bottom of the current one.
 
         Parameters
@@ -665,7 +796,7 @@ class DeltaGenerator(
             st_method_name = "_" + self._cursor.props["delta_type"]
             st_method = getattr(self, st_method_name)
             st_method(data, **kwargs)
-            return
+            return None
 
         data, self._cursor.props["last_index"] = _maybe_melt_data_for_add_rows(
             data, self._cursor.props["delta_type"], self._cursor.props["last_index"]
@@ -688,26 +819,26 @@ class DeltaGenerator(
         return self
 
 
-def _maybe_melt_data_for_add_rows(data, delta_type, last_index):
+DFT = TypeVar("DFT", bound=type_util.DataFrameCompatible)
+
+
+def _maybe_melt_data_for_add_rows(
+    data: DFT,
+    delta_type: str,
+    last_index: Any,
+) -> Tuple[Union[DFT, "DataFrame"], Union[int, Any]]:
     import pandas as pd
 
-    # For some delta types we have to reshape the data structure
-    # otherwise the input data and the actual data used
-    # by vega_lite will be different and it will throw an error.
-    if (
-        delta_type in DELTA_TYPES_THAT_MELT_DATAFRAMES
-        or delta_type in ARROW_DELTA_TYPES_THAT_MELT_DATAFRAMES
-    ):
-        if not isinstance(data, pd.DataFrame):
-            data = type_util.convert_anything_to_df(data)
-
-        if type(data.index) is pd.RangeIndex:
-            old_step = _get_pandas_index_attr(data, "step")
+    def _melt_data(
+        df: "DataFrame", last_index: Any
+    ) -> Tuple["DataFrame", Union[int, Any]]:
+        if isinstance(df.index, pd.RangeIndex):
+            old_step = _get_pandas_index_attr(df, "step")
 
             # We have to drop the predefined index
-            data = data.reset_index(drop=True)
+            df = df.reset_index(drop=True)
 
-            old_stop = _get_pandas_index_attr(data, "stop")
+            old_stop = _get_pandas_index_attr(df, "stop")
 
             if old_step is None or old_stop is None:
                 raise StreamlitAPIException(
@@ -717,23 +848,68 @@ def _maybe_melt_data_for_add_rows(data, delta_type, last_index):
             start = last_index + old_step
             stop = last_index + old_step + old_stop
 
-            data.index = pd.RangeIndex(start=start, stop=stop, step=old_step)
+            df.index = pd.RangeIndex(start=start, stop=stop, step=old_step)
             last_index = stop - 1
 
-        index_name = data.index.name
+        index_name = df.index.name
         if index_name is None:
             index_name = "index"
 
-        data = pd.melt(data.reset_index(), id_vars=[index_name])
+        df = pd.melt(df.reset_index(), id_vars=[index_name])
+        return df, last_index
+
+    # For some delta types we have to reshape the data structure
+    # otherwise the input data and the actual data used
+    # by vega_lite will be different, and it will throw an error.
+    if (
+        delta_type in DELTA_TYPES_THAT_MELT_DATAFRAMES
+        or delta_type in ARROW_DELTA_TYPES_THAT_MELT_DATAFRAMES
+    ):
+        if not isinstance(data, pd.DataFrame):
+            return _melt_data(
+                df=type_util.convert_anything_to_df(data),
+                last_index=last_index,
+            )
+        else:
+            return _melt_data(df=data, last_index=last_index)
 
     return data, last_index
 
 
-def _get_pandas_index_attr(data, attr):
+def _get_pandas_index_attr(
+    data: "Union[DataFrame, Series]",
+    attr: str,
+) -> Optional[Any]:
     return getattr(data.index, attr, None)
 
 
-def _value_or_dg(value, dg):
+@overload
+def _value_or_dg(value: None, dg: DG) -> DG:
+    ...
+
+
+@overload
+def _value_or_dg(value: Type[NoValue], dg: DG) -> None:  # type: ignore[misc]
+    ...
+
+
+@overload
+def _value_or_dg(value: Value, dg: DG) -> Value:
+    # This overload definition technically overlaps with the one above (Value
+    # contains Type[NoValue]), and since the return types are conflicting,
+    # mypy complains. Hence, the ignore-comment above. But, in practice, since
+    # the overload above is more specific, and is matched first, there is no
+    # actual overlap. The `Value` type here is thus narrowed to the cases
+    # where value is neither None nor NoValue.
+
+    # The ignore-comment should thus be fine.
+    ...
+
+
+def _value_or_dg(
+    value: Union[None, Type[NoValue], Value],
+    dg: DG,
+) -> Union[DG, None, Value]:
     """Return either value, or None, or dg.
 
     This is needed because Widgets have meaningful return values. This is
@@ -749,10 +925,10 @@ def _value_or_dg(value, dg):
         return None
     if value is None:
         return dg
-    return value
+    return cast(Value, value)
 
 
-def _enqueue_message(msg):
+def _enqueue_message(msg: ForwardMsg_pb2.ForwardMsg) -> None:
     """Enqueues a ForwardMsg proto to send to the app."""
     ctx = get_script_run_ctx()
 
