@@ -13,34 +13,38 @@
 # limitations under the License.
 
 import io
-from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
-
-from streamlit.type_util import Key, to_key
-from typing import cast, Optional, Union, BinaryIO, TextIO
+from typing import cast, Optional, Union, BinaryIO, TextIO, TYPE_CHECKING
 from textwrap import dedent
+from typing_extensions import Final
 
 import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.Button_pb2 import Button as ButtonProto
-from streamlit.in_memory_file_manager import in_memory_file_manager
+from streamlit.runtime.in_memory_file_manager import in_memory_file_manager
 from streamlit.proto.DownloadButton_pb2 import DownloadButton as DownloadButtonProto
-from streamlit.state.session_state import (
+from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
+from streamlit.runtime.state import (
+    register_widget,
     WidgetArgs,
     WidgetCallback,
     WidgetKwargs,
 )
-from streamlit.state.widgets import register_widget
+from streamlit.type_util import Key, to_key
+
 from .form import current_form_id, is_in_form
 from .utils import check_callback_rules, check_session_state_rules
 
+if TYPE_CHECKING:
+    from streamlit.delta_generator import DeltaGenerator
 
-FORM_DOCS_INFO = """
+
+FORM_DOCS_INFO: Final = """
 
 For more information, refer to the
 [documentation for forms](https://docs.streamlit.io/library/api-reference/control-flow/st.form).
 """
 
-DownloadButtonDataType = Union[str, bytes, TextIO, BinaryIO]
+DownloadButtonDataType = Union[str, bytes, TextIO, BinaryIO, io.RawIOBase]
 
 
 class ButtonMixin:
@@ -93,7 +97,7 @@ class ButtonMixin:
         ...     st.write('Goodbye')
 
         .. output::
-           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.button.py
+           https://doc-buton.streamlitapp.com/
            height: 220px
 
         """
@@ -215,7 +219,7 @@ class ButtonMixin:
         ...           )
 
         .. output::
-           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.download_button.py
+           https://doc-download-buton.streamlitapp.com/
            height: 335px
 
         """
@@ -261,7 +265,6 @@ class ButtonMixin:
 
         download_button_proto.label = label
         download_button_proto.default = False
-        download_button_proto.disabled = disabled
         marshall_file(
             self.dg._get_delta_path_str(), data, download_button_proto, mime, file_name
         )
@@ -269,10 +272,10 @@ class ButtonMixin:
         if help is not None:
             download_button_proto.help = dedent(help)
 
-        def deserialize_button(ui_value, widget_id=""):
+        def deserialize_button(ui_value: Optional[bool], widget_id: str = "") -> bool:
             return ui_value or False
 
-        current_value, _ = register_widget(
+        button_state = register_widget(
             "download_button",
             download_button_proto,
             user_key=key,
@@ -283,8 +286,13 @@ class ButtonMixin:
             serializer=bool,
             ctx=ctx,
         )
+
+        # This needs to be done after register_widget because we don't want
+        # the following proto fields to affect a widget's ID.
+        download_button_proto.disabled = disabled
+
         self.dg._enqueue("download_button", download_button_proto)
-        return cast(bool, current_value)
+        return button_state.value
 
     def _button(
         self,
@@ -307,7 +315,7 @@ class ButtonMixin:
         # for the "Form Submitter" button that's automatically created in
         # every form). We throw an error to warn the user about this.
         # We omit this check for scripts running outside streamlit, because
-        # they will have no report_ctx.
+        # they will have no script_run_ctx.
         if streamlit._is_running_with_streamlit:
             if is_in_form(self.dg) and not is_form_submitter:
                 raise StreamlitAPIException(
@@ -323,14 +331,13 @@ class ButtonMixin:
         button_proto.default = False
         button_proto.is_form_submitter = is_form_submitter
         button_proto.form_id = current_form_id(self.dg)
-        button_proto.disabled = disabled
         if help is not None:
             button_proto.help = dedent(help)
 
-        def deserialize_button(ui_value: bool, widget_id: str = "") -> bool:
+        def deserialize_button(ui_value: Optional[bool], widget_id: str = "") -> bool:
             return ui_value or False
 
-        current_value, _ = register_widget(
+        button_state = register_widget(
             "button",
             button_proto,
             user_key=key,
@@ -341,38 +348,60 @@ class ButtonMixin:
             serializer=bool,
             ctx=ctx,
         )
+
+        # This needs to be done after register_widget because we don't want
+        # the following proto fields to affect a widget's ID.
+        button_proto.disabled = disabled
+
         self.dg._enqueue("button", button_proto)
-        return cast(bool, current_value)
+
+        return button_state.value
 
     @property
-    def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
+    def dg(self) -> "DeltaGenerator":
         """Get our DeltaGenerator."""
-        return cast("streamlit.delta_generator.DeltaGenerator", self)
+        return cast("DeltaGenerator", self)
 
 
-def marshall_file(coordinates, data, proto_download_button, mimetype, file_name=None):
+def marshall_file(
+    coordinates: str,
+    data: DownloadButtonDataType,
+    proto_download_button: DownloadButtonProto,
+    mimetype: Optional[str],
+    file_name: Optional[str] = None,
+) -> None:
+    data_as_bytes: bytes
     if isinstance(data, str):
-        data = data.encode()
+        data_as_bytes = data.encode()
         mimetype = mimetype or "text/plain"
     elif isinstance(data, io.TextIOWrapper):
         string_data = data.read()
-        data = string_data.encode()
+        data_as_bytes = string_data.encode()
         mimetype = mimetype or "text/plain"
     # Assume bytes; try methods until we run out.
     elif isinstance(data, bytes):
+        data_as_bytes = data
         mimetype = mimetype or "application/octet-stream"
     elif isinstance(data, io.BytesIO):
         data.seek(0)
-        data = data.getvalue()
+        data_as_bytes = data.getvalue()
         mimetype = mimetype or "application/octet-stream"
-    elif isinstance(data, io.RawIOBase) or isinstance(data, io.BufferedReader):
+    elif isinstance(data, io.BufferedReader):
         data.seek(0)
-        data = data.read()
+        data_as_bytes = data.read()
+        mimetype = mimetype or "application/octet-stream"
+    elif isinstance(data, io.RawIOBase):
+        data.seek(0)
+        data_as_bytes = data.read() or b""
         mimetype = mimetype or "application/octet-stream"
     else:
         raise RuntimeError("Invalid binary data format: %s" % type(data))
 
     this_file = in_memory_file_manager.add(
-        data, mimetype, coordinates, file_name=file_name, is_for_static_download=True
+        data_as_bytes,
+        mimetype,
+        coordinates,
+        file_name=file_name,
+        is_for_static_download=True,
     )
     proto_download_button.url = this_file.url
