@@ -14,16 +14,15 @@
 
 from streamlit.type_util import Key, to_key
 from textwrap import dedent
-from typing import Optional, cast, List
+from typing import Optional, cast, List, TYPE_CHECKING
 
-import streamlit
 from streamlit.proto.CameraInput_pb2 import (
     CameraInput as CameraInputProto,
 )
 
-from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
-from streamlit.state.widgets import register_widget
-from streamlit.state.session_state import (
+from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
+from streamlit.runtime.state import (
+    register_widget,
     WidgetArgs,
     WidgetCallback,
     WidgetKwargs,
@@ -33,11 +32,14 @@ from ..proto.Common_pb2 import (
     FileUploaderState as FileUploaderStateProto,
     UploadedFileInfo as UploadedFileInfoProto,
 )
-from ..uploaded_file_manager import UploadedFile, UploadedFileRec
+from streamlit.runtime.uploaded_file_manager import UploadedFile, UploadedFileRec
 
 from .form import current_form_id
 from .utils import check_callback_rules, check_session_state_rules
 
+if TYPE_CHECKING:
+    from streamlit.delta_generator import DeltaGenerator
+
 SomeUploadedSnapshotFile = Optional[UploadedFile]
 
 
@@ -132,7 +134,6 @@ class CameraInputMixin:
         camera_input_proto = CameraInputProto()
         camera_input_proto.label = label
         camera_input_proto.form_id = current_form_id(self.dg)
-        camera_input_proto.disabled = disabled
 
         if help is not None:
             camera_input_proto.help = dedent(help)
@@ -172,7 +173,7 @@ class CameraInputMixin:
                 return_value = UploadedFile(file_recs[0])
             return return_value
 
-        widget_value, _ = register_widget(
+        camera_input_state = register_widget(
             "camera_input",
             camera_input_proto,
             user_key=key,
@@ -184,8 +185,14 @@ class CameraInputMixin:
             ctx=ctx,
         )
 
+        # This needs to be done after register_widget because we don't want
+        # the following proto fields to affect a widget's ID.
+        camera_input_proto.disabled = disabled
+
         ctx = get_script_run_ctx()
-        camera_image_input_state = serialize_camera_image_input(widget_value)
+        camera_image_input_state = serialize_camera_image_input(
+            camera_input_state.value
+        )
 
         uploaded_shapshot_info = camera_image_input_state.uploaded_file_info
 
@@ -201,12 +208,12 @@ class CameraInputMixin:
             )
 
         self.dg._enqueue("camera_input", camera_input_proto)
-        return cast(SomeUploadedSnapshotFile, widget_value)
+        return camera_input_state.value
 
     @property
-    def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
+    def dg(self) -> "DeltaGenerator":
         """Get our DeltaGenerator."""
-        return cast("streamlit.delta_generator.DeltaGenerator", self)
+        return cast("DeltaGenerator", self)
 
     @staticmethod
     def _get_file_recs_for_camera_input_widget(
