@@ -12,15 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
+from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
 from streamlit.type_util import Key, to_key
 from textwrap import dedent
-from typing import cast, Optional
+from typing import cast, Optional, TYPE_CHECKING
 
-import streamlit
 from streamlit.proto.Checkbox_pb2 import Checkbox as CheckboxProto
-from streamlit.state.widgets import register_widget
-from streamlit.state.session_state import (
+from streamlit.runtime.state import (
+    register_widget,
     WidgetArgs,
     WidgetCallback,
     WidgetKwargs,
@@ -29,6 +28,10 @@ from .form import current_form_id
 from .utils import check_callback_rules, check_session_state_rules
 
 
+if TYPE_CHECKING:
+    from streamlit.delta_generator import DeltaGenerator
+
+
 class CheckboxMixin:
     def checkbox(
         self,
@@ -81,7 +84,7 @@ class CheckboxMixin:
         ...     st.write('Great!')
 
         .. output::
-           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.checkbox.py
+           https://doc-checkbox.streamlitapp.com/
            height: 220px
 
         """
@@ -121,14 +124,13 @@ class CheckboxMixin:
         checkbox_proto.label = label
         checkbox_proto.default = bool(value)
         checkbox_proto.form_id = current_form_id(self.dg)
-        checkbox_proto.disabled = disabled
         if help is not None:
             checkbox_proto.help = dedent(help)
 
         def deserialize_checkbox(ui_value: Optional[bool], widget_id: str = "") -> bool:
             return bool(ui_value if ui_value is not None else value)
 
-        current_value, set_frontend_value = register_widget(
+        checkbox_state = register_widget(
             "checkbox",
             checkbox_proto,
             user_key=key,
@@ -140,14 +142,17 @@ class CheckboxMixin:
             ctx=ctx,
         )
 
-        if set_frontend_value:
-            checkbox_proto.value = current_value
+        # This needs to be done after register_widget because we don't want
+        # the following proto fields to affect a widget's ID.
+        checkbox_proto.disabled = disabled
+        if checkbox_state.value_changed:
+            checkbox_proto.value = checkbox_state.value
             checkbox_proto.set_value = True
 
         self.dg._enqueue("checkbox", checkbox_proto)
-        return cast(bool, current_value)
+        return checkbox_state.value
 
     @property
-    def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
+    def dg(self) -> "DeltaGenerator":
         """Get our DeltaGenerator."""
-        return cast("streamlit.delta_generator.DeltaGenerator", self)
+        return cast("DeltaGenerator", self)
