@@ -14,9 +14,9 @@
 
 from typing import cast, Optional, TYPE_CHECKING, Union
 
-from streamlit import type_util
 from streamlit.proto.Markdown_pb2 import Markdown as MarkdownProto
-from .utils import clean_text
+from streamlit.string_util import clean_text
+from streamlit.type_util import SupportsStr, is_sympy_expession
 
 if TYPE_CHECKING:
     import sympy
@@ -25,7 +25,9 @@ if TYPE_CHECKING:
 
 
 class MarkdownMixin:
-    def markdown(self, body: str, unsafe_allow_html: bool = False) -> "DeltaGenerator":
+    def markdown(
+        self, body: SupportsStr, unsafe_allow_html: bool = False
+    ) -> "DeltaGenerator":
         """Display string formatted as Markdown.
 
         Parameters
@@ -78,58 +80,9 @@ class MarkdownMixin:
 
         return self.dg._enqueue("markdown", markdown_proto)
 
-    def header(self, body: str, anchor: Optional[str] = None) -> "DeltaGenerator":
-        """Display text in header formatting.
-
-        Parameters
-        ----------
-        body : str
-            The text to display.
-
-        anchor : str
-            The anchor name of the header that can be accessed with #anchor
-            in the URL. If omitted, it generates an anchor using the body.
-
-        Example
-        -------
-        >>> st.header('This is a header')
-
-        """
-        header_proto = MarkdownProto()
-        if anchor is None:
-            header_proto.body = f"## {clean_text(body)}"
-        else:
-            header_proto.body = f'<h2 data-anchor="{anchor}">{clean_text(body)}</h2>'
-            header_proto.allow_html = True
-        return self.dg._enqueue("markdown", header_proto)
-
-    def subheader(self, body: str, anchor: Optional[str] = None) -> "DeltaGenerator":
-        """Display text in subheader formatting.
-
-        Parameters
-        ----------
-        body : str
-            The text to display.
-
-        anchor : str
-            The anchor name of the header that can be accessed with #anchor
-            in the URL. If omitted, it generates an anchor using the body.
-
-        Example
-        -------
-        >>> st.subheader('This is a subheader')
-
-        """
-        subheader_proto = MarkdownProto()
-        if anchor is None:
-            subheader_proto.body = f"### {clean_text(body)}"
-        else:
-            subheader_proto.body = f'<h3 data-anchor="{anchor}">{clean_text(body)}</h3>'
-            subheader_proto.allow_html = True
-
-        return self.dg._enqueue("markdown", subheader_proto)
-
-    def code(self, body: str, language: Optional[str] = "python") -> "DeltaGenerator":
+    def code(
+        self, body: SupportsStr, language: Optional[str] = "python"
+    ) -> "DeltaGenerator":
         """Display a code block with optional syntax highlighting.
 
         (This is a convenience wrapper around `st.markdown()`)
@@ -155,42 +108,13 @@ class MarkdownMixin:
 
         """
         code_proto = MarkdownProto()
-        markdown = "```%(language)s\n%(body)s\n```" % {
-            "language": language or "",
-            "body": body,
-        }
+        markdown = f'```{language or ""}\n{body}\n```'
         code_proto.body = clean_text(markdown)
         return self.dg._enqueue("markdown", code_proto)
 
-    def title(self, body: str, anchor: Optional[str] = None) -> "DeltaGenerator":
-        """Display text in title formatting.
-
-        Each document should have a single `st.title()`, although this is not
-        enforced.
-
-        Parameters
-        ----------
-        body : str
-            The text to display.
-
-        anchor : str
-            The anchor name of the header that can be accessed with #anchor
-            in the URL. If omitted, it generates an anchor using the body.
-
-        Example
-        -------
-        >>> st.title('This is a title')
-
-        """
-        title_proto = MarkdownProto()
-        if anchor is None:
-            title_proto.body = f"# {clean_text(body)}"
-        else:
-            title_proto.body = f'<h1 data-anchor="{anchor}">{clean_text(body)}</h1>'
-            title_proto.allow_html = True
-        return self.dg._enqueue("markdown", title_proto)
-
-    def caption(self, body: str, unsafe_allow_html: bool = False) -> "DeltaGenerator":
+    def caption(
+        self, body: SupportsStr, unsafe_allow_html: bool = False
+    ) -> "DeltaGenerator":
         """Display text in small font.
 
         This should be used for captions, asides, footnotes, sidenotes, and
@@ -233,7 +157,7 @@ class MarkdownMixin:
         caption_proto.is_caption = True
         return self.dg._enqueue("markdown", caption_proto)
 
-    def latex(self, body: Union[str, "sympy.Expr"]) -> "DeltaGenerator":
+    def latex(self, body: Union[SupportsStr, "sympy.Expr"]) -> "DeltaGenerator":
         # This docstring needs to be "raw" because of the backslashes in the
         # example below.
         r"""Display mathematical expressions formatted as LaTeX.
@@ -258,7 +182,7 @@ class MarkdownMixin:
         ...     ''')
 
         """
-        if type_util.is_sympy_expession(body):
+        if is_sympy_expession(body):
             import sympy
 
             body = sympy.latex(body)
