@@ -19,8 +19,7 @@ from typing_extensions import TypeAlias, Literal
 
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.Metric_pb2 import Metric as MetricProto
-
-from .utils import clean_text
+from streamlit.string_util import clean_text
 
 if TYPE_CHECKING:
     import numpy as np
