@@ -18,13 +18,13 @@ from typing import Any, Callable, Optional, cast
 import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.Slider_pb2 import Slider as SliderProto
-from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
-from streamlit.state.session_state import (
+from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
+from streamlit.runtime.state import (
+    register_widget,
     WidgetArgs,
     WidgetCallback,
     WidgetKwargs,
 )
-from streamlit.state.widgets import register_widget
 from streamlit.type_util import Key, OptionSequence, ensure_indexable, to_key
 from streamlit.util import index_
 from .form import current_form_id
@@ -35,7 +35,7 @@ class SelectSliderMixin:
     def select_slider(
         self,
         label: str,
-        options: OptionSequence = [],
+        options: OptionSequence = (),
         value: Any = None,
         format_func: Callable[[Any], Any] = str,
         key: Optional[Key] = None,
@@ -114,7 +114,7 @@ class SelectSliderMixin:
         >>> st.write('You selected wavelengths between', start_color, 'and', end_color)
 
         .. output::
-           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.select_slider.py
+           https://doc-select-slider.streamlitapp.com/
            height: 450px
 
         """
@@ -136,7 +136,7 @@ class SelectSliderMixin:
     def _select_slider(
         self,
         label: str,
-        options: OptionSequence = [],
+        options: OptionSequence = (),
         value: Any = None,
         format_func: Callable[[Any], Any] = str,
         key: Optional[Key] = None,
@@ -189,7 +189,6 @@ class SelectSliderMixin:
         slider_proto.data_type = SliderProto.INT
         slider_proto.options[:] = [str(format_func(option)) for option in opt]
         slider_proto.form_id = current_form_id(self.dg)
-        slider_proto.disabled = disabled
         if help is not None:
             slider_proto.help = dedent(help)
 
@@ -207,7 +206,7 @@ class SelectSliderMixin:
         def serialize_select_slider(v):
             return as_index_list(v)
 
-        current_value, set_frontend_value = register_widget(
+        widget_state = register_widget(
             "slider",
             slider_proto,
             user_key=key,
@@ -219,12 +218,15 @@ class SelectSliderMixin:
             ctx=ctx,
         )
 
-        if set_frontend_value:
-            slider_proto.value[:] = serialize_select_slider(current_value)
+        # This needs to be done after register_widget because we don't want
+        # the following proto fields to affect a widget's ID.
+        slider_proto.disabled = disabled
+        if widget_state.value_changed:
+            slider_proto.value[:] = serialize_select_slider(widget_state.value)
             slider_proto.set_value = True
 
         self.dg._enqueue("slider", slider_proto)
-        return current_value
+        return widget_state.value
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
