@@ -13,7 +13,7 @@
 # limitations under the License.
 
 from datetime import date, time, datetime, timedelta, timezone
-from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
+from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
 from streamlit.type_util import Key, to_key
 from typing import Any, List, cast, Optional
 from textwrap import dedent
@@ -23,12 +23,13 @@ from streamlit.errors import StreamlitAPIException
 from streamlit.js_number import JSNumber
 from streamlit.js_number import JSNumberBoundsException
 from streamlit.proto.Slider_pb2 import Slider as SliderProto
-from streamlit.state.session_state import (
+from streamlit.runtime.state import (
+    get_session_state,
+    register_widget,
     WidgetArgs,
     WidgetCallback,
     WidgetKwargs,
 )
-from streamlit.state.widgets import register_widget
 from .form import current_form_id
 from .utils import check_callback_rules, check_session_state_rules
 
@@ -41,7 +42,7 @@ class SliderMixin:
         max_value=None,
         value=None,
         step=None,
-        format=None,
+        format: Optional[str] = None,
         key: Optional[Key] = None,
         help: Optional[str] = None,
         on_change: Optional[WidgetCallback] = None,
@@ -144,7 +145,7 @@ class SliderMixin:
         >>> st.write("Start time:", start_time)
 
         .. output::
-           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.slider.py
+           https://doc-slider.streamlitapp.com/
            height: 300px
 
         """
@@ -186,9 +187,18 @@ class SliderMixin:
         check_callback_rules(self.dg, on_change)
         check_session_state_rules(default_value=value, key=key)
 
-        # Set value default.
         if value is None:
-            value = min_value if min_value is not None else 0
+            # Set value from session_state if exists.
+            session_state = get_session_state().filtered_state
+
+            # we look first to session_state value of the widget because
+            # depending on the value (single value or list/tuple) the slider should be
+            # initializing differently (either as range or single value slider)
+            if key is not None and key in session_state:
+                value = session_state[key]
+            else:
+                # Set value default.
+                value = min_value if min_value is not None else 0
 
         SUPPORTED_TYPES = {
             int: SliderProto.INT,
@@ -274,14 +284,10 @@ class SliderMixin:
             max_value = DEFAULTS[data_type]["max_value"]
         if step is None:
             step = DEFAULTS[data_type]["step"]
-            if (
-                data_type
-                in (
-                    SliderProto.DATETIME,
-                    SliderProto.DATE,
-                )
-                and max_value - min_value < timedelta(days=1)
-            ):
+            if data_type in (
+                SliderProto.DATETIME,
+                SliderProto.DATE,
+            ) and max_value - min_value < timedelta(days=1):
                 step = timedelta(minutes=15)
         if format is None:
             format = DEFAULTS[data_type]["format"]
@@ -405,8 +411,11 @@ class SliderMixin:
             UTC_EPOCH = datetime(1970, 1, 1, tzinfo=timezone.utc)
 
             def _datetime_to_micros(dt):
-                # If dt is naive, Python converts from local time
-                utc_dt = dt.astimezone(timezone.utc)
+                # The frontend is not aware of timezones and only expects a UTC-based timestamp (in microseconds).
+                # Since we want to show the date/time exactly as it is in the given datetime object,
+                # we just set the tzinfo to UTC and do not do any timezone conversions.
+                # Only the backend knows about original timezone and will replace the UTC timestamp in the deserialization.
+                utc_dt = dt.replace(tzinfo=timezone.utc)
                 return _delta_to_micros(utc_dt - UTC_EPOCH)
 
             # Restore times/datetimes to original timezone (dates are always naive)
@@ -418,8 +427,9 @@ class SliderMixin:
 
             def _micros_to_datetime(micros):
                 utc_dt = UTC_EPOCH + timedelta(microseconds=micros)
-                # Convert from utc back to original time (local time if naive)
-                return utc_dt.astimezone(orig_tz).replace(tzinfo=orig_tz)
+                # Add the original timezone. No conversion is required here,
+                # since in the serialization, we also just replace the timestamp with UTC.
+                return utc_dt.replace(tzinfo=orig_tz)
 
             value = list(map(_datetime_to_micros, value))
             min_value = _datetime_to_micros(min_value)
@@ -441,7 +451,6 @@ class SliderMixin:
         slider_proto.data_type = data_type
         slider_proto.options[:] = []
         slider_proto.form_id = current_form_id(self.dg)
-        slider_proto.disabled = disabled
         if help is not None:
             slider_proto.help = dedent(help)
 
@@ -477,7 +486,7 @@ class SliderMixin:
                 value = [_datetime_to_micros(v) for v in value]
             return value
 
-        current_value, set_frontend_value = register_widget(
+        widget_state = register_widget(
             "slider",
             slider_proto,
             user_key=key,
@@ -489,12 +498,15 @@ class SliderMixin:
             ctx=ctx,
         )
 
-        if set_frontend_value:
-            slider_proto.value[:] = serialize_slider(current_value)
+        # This needs to be done after register_widget because we don't want
+        # the following proto fields to affect a widget's ID.
+        slider_proto.disabled = disabled
+        if widget_state.value_changed:
+            slider_proto.value[:] = serialize_slider(widget_state.value)
             slider_proto.set_value = True
 
         self.dg._enqueue("slider", slider_proto)
-        return current_value
+        return widget_state.value
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
