@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,15 +14,16 @@
 
 import tempfile
 import shutil
+from streamlit import util
 
 # We provide our own context manager for temporary directory that wraps
 # tempfile.mkdtemp
 
 
 class TemporaryDirectory(object):
-    """Temporary directory context mananger.
+    """Temporary directory context manager.
 
-    Creates a temporary directory that exists whithin the context manager scope.
+    Creates a temporary directory that exists within the context manager scope.
     It returns the path to the created directory.
     Wrapper on top of tempfile.mkdtemp.
 
@@ -41,6 +42,9 @@ class TemporaryDirectory(object):
         self._args = args
         self._kwargs = kwargs
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     def __enter__(self):
         self._path = tempfile.mkdtemp(*self._args, **self._kwargs)
         return self._path
