@@ -23,10 +23,10 @@ from streamlit import file_util
 from streamlit.folder_black_list import FolderBlackList
 
 from streamlit.logger import get_logger
-from streamlit.session_data import SessionData
-from streamlit.watcher.file_watcher import (
-    get_default_file_watcher_class,
-    NoOpFileWatcher,
+from streamlit.source_util import get_pages
+from streamlit.watcher.path_watcher import (
+    get_default_path_watcher_class,
+    NoOpPathWatcher,
 )
 
 LOGGER = get_logger(__name__)
@@ -35,14 +35,16 @@ WatchedModule = collections.namedtuple("WatchedModule", ["watcher", "module_name
 
 # This needs to be initialized lazily to avoid calling config.get_option() and
 # thus initializing config options when this file is first imported.
-FileWatcher = None
+PathWatcher = None
 
 
 class LocalSourcesWatcher:
-    def __init__(self, session_data: SessionData):
-        self._session_data = session_data
-        self._on_file_changed: List[Callable[[], None]] = []
+    def __init__(self, main_script_path: str):
+        self._main_script_path = os.path.abspath(main_script_path)
+        self._script_folder = os.path.dirname(self._main_script_path)
+        self._on_file_changed: List[Callable[[str], None]] = []
         self._is_closed = False
+        self._cached_sys_modules: Set[str] = set()
 
         # Blacklist for folders that should not be watched
         self._folder_black_list = FolderBlackList(
@@ -51,12 +53,13 @@ class LocalSourcesWatcher:
 
         self._watched_modules: Dict[str, WatchedModule] = {}
 
-        self._register_watcher(
-            self._session_data.script_path,
-            module_name=None,  # Only the root script has None here.
-        )
+        for page_info in get_pages(self._main_script_path).values():
+            self._register_watcher(
+                page_info["script_path"],
+                module_name=None,  # Only root scripts have their modules set to None
+            )
 
-    def register_file_change_callback(self, cb: Callable[[], None]) -> None:
+    def register_file_change_callback(self, cb: Callable[[str], None]) -> None:
         self._on_file_changed.append(cb)
 
     def on_file_changed(self, filepath):
@@ -81,7 +84,7 @@ class LocalSourcesWatcher:
                 del sys.modules[wm.module_name]
 
         for cb in self._on_file_changed:
-            cb()
+            cb(filepath)
 
     def close(self):
         for wm in self._watched_modules.values():
@@ -90,16 +93,16 @@ class LocalSourcesWatcher:
         self._is_closed = True
 
     def _register_watcher(self, filepath, module_name):
-        global FileWatcher
-        if FileWatcher is None:
-            FileWatcher = get_default_file_watcher_class()
+        global PathWatcher
+        if PathWatcher is None:
+            PathWatcher = get_default_path_watcher_class()
 
-        if FileWatcher is NoOpFileWatcher:
+        if PathWatcher is NoOpPathWatcher:
             return
 
         try:
             wm = WatchedModule(
-                watcher=FileWatcher(filepath, self.on_file_changed),
+                watcher=PathWatcher(filepath, self.on_file_changed),
                 module_name=module_name,
             )
         except PermissionError:
@@ -113,7 +116,7 @@ class LocalSourcesWatcher:
         if filepath not in self._watched_modules:
             return
 
-        if filepath == self._session_data.script_path:
+        if filepath == self._main_script_path:
             return
 
         wm = self._watched_modules[filepath]
@@ -126,7 +129,7 @@ class LocalSourcesWatcher:
     def _file_should_be_watched(self, filepath):
         # Using short circuiting for performance.
         return self._file_is_new(filepath) and (
-            file_util.file_is_in_folder_glob(filepath, self._session_data.script_folder)
+            file_util.file_is_in_folder_glob(filepath, self._script_folder)
             or file_util.file_in_pythonpath(filepath)
         )
 
@@ -134,12 +137,13 @@ class LocalSourcesWatcher:
         if self._is_closed:
             return
 
-        modules_paths = {
-            name: self._exclude_blacklisted_paths(get_module_paths(module))
-            for name, module in dict(sys.modules).items()
-        }
-
-        self._register_necessary_watchers(modules_paths)
+        if set(sys.modules) != self._cached_sys_modules:
+            modules_paths = {
+                name: self._exclude_blacklisted_paths(get_module_paths(module))
+                for name, module in dict(sys.modules).items()
+            }
+            self._cached_sys_modules = set(sys.modules)
+            self._register_necessary_watchers(modules_paths)
 
     def _register_necessary_watchers(self, module_paths: Dict[str, Set[str]]) -> None:
         for name, paths in module_paths.items():
